<?php

/***
 * $oDatabase = new Database('name', 'host');
 * $oDatabase->connect('username', 'password');
 ***/

class Database {

	// DATA MEMBERS

	/***
	 * The name of the database
	 * @type string
	 ***/
	var $name;

	/***
	 * The host the database is on; if left empty, sockets are used
	 * @type string
	 ***/
	var $host;

	/***
	 * The internal link to the database
	 * @type int
	 ***/
	var $link;

	// CREATORS

    function &getConnected($aConf = array()) {
        static $aInstances;
        if (! isSet($aInstances)) $aInstances = array();

        $sSignature = serialize($aConf);
        if (! isSet($aInstances[$sSignature])) {
            $aInstances[$sSignature] = new Database($aConf['database'], $aConf['hostname']);
        }
        if (! $aInstances[$sSignature]->isConnected()) {
			$aInstances[$sSignature]->connect($aConf['username'], $aConf['password']);
    	}
        return $aInstances[$sSignature];
    }

	/***
	 * Prepare a new database connection on the specified host. If the
	 * host is left empty, the localhost is used. To use sockets, specify
	 * the path to the socket as the hostname (prepended with ':')
	 * @param $name the name of the database
	 * @param $host the host the database is running on
	 ***/
	function Database($name, $host = "") {
		$this->name = $name;
		$this->host = $host;
		$this->link = 0;
	}

	// MANIPULATORS

	/***
	 * Set the internal link identifier; this is a protected method.
	 * @param $link the resource index
	 * @returns void
	 * @protected
	 ***/
	function setLink($link) {
		$this->link = $link;
	}

	 /***
	 * Make a connection with a database; return <code>true</code> on success,
	 * false otherwise
	 * @param $username the name of the database user
	 * @param $password the password of the database user
	 * @returns bool
	 ***/
	function connect($username, $password) {
		$this->setLink(mysql_connect($this->getHost(), $username, $password));
		mysql_select_db($this->getName(), $this->getLink());
		return $this->isConnected();
	}

	/***
	 * Close the connection with the database; this method needn't be called
	 * when using a persistent connection
	 * @returns void
	 ***/
	function disconnect() {
		mysql_close($this->getLink());
		$this->setLink(0);
	}

	/***
	 * Execute a query and return the result
	 * @param $sql the query to perform
	 * @returns resource
	 * @returns false
	 * @protected
	 ***/
	function query($sql) {
		return mysql_query($sql, $this->getLink());
	}

	/***
	 * Execute a query and return the status
	 * @param $sql the query to perform
	 * @returns boolean
	 ***/
	function execute($sql) {
		return mysql_query($sql, $this->getLink());
	}

	// ACCESSORS

	/***
	 * Get the internal link identifier; this is a protected method.
	 * @returns int
	 * @protected
	 ***/
	function getLink() {
		return $this->link;
	}

	/***
	 * Get the host the database is running on
	 * @returns string
	 ***/
	function getHost() {
		return $this->host;
	}

	/***
	 * Get the name of the database
	 * @returns string
	 ***/
	function getName() {
		return $this->name;
	}

	/***
	 * Check if a connection with the database has been established
	 * @returns bool
	 ***/
	function isConnected() {
		return ($this->link != 0);
	}

	/***
	 * Get the last error message produced by the database
	 * @returns string
	 ***/
	function getErrorMessage() {
		return mysql_error($this->getLink());
	}

	/***
	 * Get the ID generated from the previous INSERT operation
	 * @returns integer
	 ***/
	function getInsertID() {
		return mysql_insert_id($this->getLink());
	}

	/***
	 * Get the number of rows affected by the last INSERT, UPDATE or DELETE query
	 * @returns integer
	 ***/
	function getAffectedRows() {
		return mysql_affected_rows($this->getLink());
	}

}
